package com.zanxiang.sdk.common.resolver;

import com.zanxiang.common.enums.HttpStatusEnum;
import com.zanxiang.common.exception.CustomException;
import com.zanxiang.common.utils.StringUtils;
import com.zanxiang.common.utils.bean.BeanUtils;
import com.zanxiang.sdk.common.annotation.ValidLogin;
import com.zanxiang.sdk.domain.dto.UserTokenDTO;
import com.zanxiang.sdk.domain.params.UserData;
import com.zanxiang.sdk.service.IUserTokenService;
import org.springframework.core.MethodParameter;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.support.WebDataBinderFactory;
import org.springframework.web.context.request.NativeWebRequest;
import org.springframework.web.method.support.HandlerMethodArgumentResolver;
import org.springframework.web.method.support.ModelAndViewContainer;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

/**
 * token转用户信息解析器
 *
 * @author xufeng
 * @date 2022/6/20 09:59
 */
@Component
@Service
public class TokenArgumentResolver implements HandlerMethodArgumentResolver {

    @Resource
    private IUserTokenService iUserTokenService;

    @Override
    public boolean supportsParameter(MethodParameter parameter) {
        return parameter.getParameterType().equals(UserData.class);
    }

    @Override
    public Object resolveArgument(MethodParameter parameter, ModelAndViewContainer mavContainer, NativeWebRequest webRequest, WebDataBinderFactory binderFactory) throws Exception {
        HttpServletRequest request = webRequest.getNativeRequest(HttpServletRequest.class);
        if (request == null) {
            throw new CustomException(HttpStatusEnum.UNKNOWN_ERROR);
        }
        boolean b = parameter.hasParameterAnnotation(ValidLogin.class);
        String token = request.getHeader("token");
        if (b && StringUtils.isEmpty(token)) {
            throw new CustomException(HttpStatusEnum.USER_NO_LOGIN);
        }
        String deviceType = "";
        if (StringUtils.isEmpty(token)) {
            return null;
        }
        UserTokenDTO tokenInfoByTokenDevice = iUserTokenService.getTokenInfoByTokenDevice(token, deviceType);
        if (tokenInfoByTokenDevice == null) {
            return null;
        }
        if (b && tokenInfoByTokenDevice.getUserId() == null) {
            throw new CustomException(HttpStatusEnum.USER_NO_LOGIN);
        }
        return BeanUtils.copy(tokenInfoByTokenDevice, UserData.class);
    }
}
